<?php
// $Id: crmngp_ui.inc,v 1.31.2.30 2009/07/31 17:37:45 seanr Exp $

/**
 * @file
 * Enables users to interact with the NGP CRM SOAP API (requires crmngp).
 */

/**
 * Implementation of hook_help().
 */
function crmngp_ui_help($section) {
  switch ($section) {
    case 'admin/modules#description':
      return t('Enables users to interact with the NGP CRM SOAP API (requires crmngp).');

    case 'crmngp_ui/contact':
      return '<p>'.t('All the contacts that have been submitted are displayed below.').'</p>';
    case 'crmngp_ui/contact/add':
      return '<p>'.t('Add a new contact by filling in the fields below.').'</p>';
  }
}

/**
 * Menu callback; displays a list of contacts stored in NGP.
 */
function crmngp_ui_contact_overview() {

  // our headers are the required fields - no point in displaying
  // a lot of empty columns for the optionally submitted stuff.
  $headers = array(t('ID'));
  $available_fields = crmngp_crm_contact_field_labels();
  $required_fields  = variable_get('crmngp_crm_contactadd_required_fields', array());
  foreach ($required_fields as $required_field) {
    $headers[] = $available_fields[$required_field];
  }
  $headers[] = t('Operations');

  // load in every user...
  $page_increment = 50; // ...50 at a time.
  $start_from = $_GET['page'] ? $_GET['page'] : 0;
  $contacts = crmngp_crm_contact_load(array(), $start_from + 1, $page_increment);

  // and display the required fields.
  foreach ($contacts['contacts'] as $contact) {
    $row = array(); // storage, storage!
    $row[] = $contact['contact_id'];
    foreach ($required_fields as $required_field) {
      $row[] = $contact[$required_field];
    } // now we have to show our little edit link for the Operations column.
    $row[] = l('edit', 'crmngp_ui/contact/'.$contact['contact_id'].'/edit');
    $rows[] = $row;
  }

  drupal_set_message(format_plural($contacts['count'], 'Found 1 result.', 'Found @count results.'));

  $GLOBALS['pager_page_array'][] = $start_from; // waiting on API changes.
  $GLOBALS['pager_total'][] = intval($contacts['count'] / $page_increment) + 1;

  $output .= theme('table', $headers, $rows);
  $output .= theme('pager', NULL, $page_increment, 0);

  return $output;
}

function _crmngp_fb_codes($code){
  $political = array (
    'Very Liberal' => 'VryLiberal',
    'Liberal' => 'Liberal',
    'Moderate' => 'Moderate',
    'Conservative' => 'Conserv',
    'Very Conservative' => 'VryConser',
    'Apathetic' => 'Apathetic',
    'Libertarian' => 'Libertar');
  return $political[$code];
}


/**
 * Contact form; displays the form for adding an individual via NGPAPI.
 *
 * @param $contact_id
 *   Optional; the contact id to edit.
 */
function crmngp_ui_contact_form($section = NULL, $contact_id = NULL, $options = NULL) {
  if (!variable_get('crmngp_crm_credentials', NULL) && !variable_get('crmngp_cwp_credentials', NULL)) {
    drupal_not_found();
  }
  //drupal_set_message($contact_id);
  drupal_add_css(drupal_get_path('module', 'crmngp') .'/crmngp.css', 'module', 'all', FALSE);
  $contact = array();


  // Fetch Field Settings for Later Use
  $settings = variable_get('crmapi_'.$section.'_fields_settings', crmngp_default_settings());

  //drupal_set_message('section: '.$section);

  /* Code for face book interatction - to access you must install facebook for drupal module and specifiy the app being used
    */

  if (module_exists('fb') && $_GET['appName']) {
    //gets facebook app name from url
    $appName = $_GET['appName'];
    //lookup app info in drupal database
    $query = "SELECT * FROM {fb_app} WHERE label = '$appName'";
    $result = db_query(db_rewrite_sql($query));
    $app = db_fetch_object($result);
    //retrieve api and secret
    $appapikey = $app->apikey;
    $appsecret = $app->secret;

    //see if an email was passed
    if($_GET['email'] != ''){
      $preEmail = $_GET['email'];
      $preContact = array (
        'email' => $preEmail,
        'contact_type' => 'I');
      //add email to COO before asking for facebook permission
      $contact_id =crmngp_crm_contact_save($preContact);
    }

    //grab facebook instance
    global $fb;
    //ask for permission to access facebook data
    $user = $fb->require_login();

    //capture facebook info

    if($user){

      $uids = array($user);
      $fields = array('first_name','last_name','current_location','hometown_location','interests','political');
      //call to facebook for user info
      $user_info = $fb->api_client->users_getInfo($uids, $fields);
      $preEmail = $_GET['email'];
      //setup contact for insertion
      $preContact = array (
        'contact_id' => $contact_id,
        'first_name' => $user_info[0]['first_name'],
        'last_name' => $user_info[0]['last_name'],
        'mail_name' => $user_info[0]['first_name'].' '.$user_info[0]['last_name'],
        'city' => $user_info[0]['current_location']['city'],
        'state' => $user_info[0]['current_location']['state'],
        'zip' => $user_info[0]['current_location']['zip'],
        'email' => $preEmail,
        'codes' => _crmngp_fb_codes($user_info[0]['political']).', Facebook',
        'contact_type' => 'I');
      //insert with facebook info

      //$contact_id =crmngp_crm_contact_save($preContact);

    }
  }

  global $user;
  if (!empty($_SESSION['contact_id'])) {
    $contact_id = $_SESSION['contact_id'];
  } else if ((!$contact_id) && ($section != 'contact') && !empty($user->crmapi_contact)) {
    $contact_id = $user->crmapi_contact;
    //drupal_set_message('<pre>'.print_r($user,1).'</pre>');
  }
  if (is_numeric($_REQUEST['a'])) {
    $amount = $_REQUEST['a'];
  } else {
    $amount = '';
  }

  $visible_fields   = variable_get('crmapi_'.$section.'_fields', array());
  $required_fields  = variable_get('crmapi_'.$section.'_fields_required', array());
  $available_fields = crmngp_crmapi_contact_fields($section);

  // Clean fields
  $available_fields = crmapi_clean_fields($available_fields);
  _debug($available_fields, 'available', 'fields');

  //dsm($available_fields);
  //dsm($visible_fields);
  
  //dsm($available_fields['amount']);
  
  if ($section == 'contribute') {
    //dsm('here');
    drupal_add_js(array('crmngp' => array('submit' => t('Submit'))), 'setting');
    drupal_add_js(drupal_get_path('module', 'crmngp') .'/crmngp.js', 'module');
    //drupal_add_js(drupal_get_path('module', 'crmngp') .'/jquery.form.js', 'module');
  }

  $form['section'] = array( '#type' => 'hidden', '#value' => $section, );

  // Fields is array for storing field code data from CWP
  $fields = null;
  if ($contact_id != '') { // if we're editing, we'll load the contact_id.
    $contact = array_shift(crmngp_crm_contact_load(array('contact_id' => $contact_id)));
    $fields = crmngp_crm_fields_get($contact_id);
    
    //print_r($contact); exit;
   
    // Only use the contact ID if the record still exists.
    if (isset($contact['contact_id'])) {
      $form['contact_id'] = array( '#type' => 'value', '#value' => $contact['contact_id'], );
    } else {
      // try to get data from user profile if it's missing from COO
      $contact = unserialize($user->data);
    }
  }

  // for every available field we have, see if it
  // should be shown and/or required for input.
  $i = -10;
  $fieldsets = array(
    'contact' => array(
      '#type' => 'fieldset', 
      '#collapsible' => FALSE, 
      '#title' => t('Contact Information'), 
      '#attributes' => array('class' => 'contact'),
    ),
    'employment' => array(
      '#type' => 'fieldset', 
		  '#collapsible' => FALSE, 
		  '#title' => t('Employer Information'), 
		  '#description' => variable_get('crmngp_employment', ''), 
		  '#attributes' => array('class' => 'employment'),
		),
    'payment' => array(
      '#type' => 'fieldset', 
      '#collapsible' => FALSE, 
      '#title' => t('Payment Information'), 
      '#attributes' => array('class' => 'payment'),
    ),
    'compliance' => array(
      '#type' => 'fieldset', 
		  '#collapsible' => FALSE, 
		  '#title' => t('Legal Compliance'), 
		  '#attributes' => array('class' => 'compliance'),
    ),
    'volunteer' => array(
      '#type' => 'fieldset', 
      '#collapsible' => FALSE, 
      '#title' => t('Volunteer Options'), 
      '#attributes' => array('class' => 'volunteer'),
    ),
    'interest' => array(
      '#type' => 'fieldset', 
			'#collapsible' => FALSE, 
			'#title' => $settings['interestcodes']['title'], 
			'#attributes' => array('class' => 'interest'),
		),
    'notes' => array(
      '#type' => 'fieldset',
      '#collapsible' => FALSE,
      '#title' => '',
      '#attributes' => array('class' => 'notes'),
    ),
  );


  foreach ($available_fields as $field_name => $field_info) {
    if (($visible_fields[$field_name] == 1) || ($field_info['#required'] == true)) {
      if (!isset($field_info['fieldset'])) {
        $form[$field_name] = $field_info;
        $form[$field_name]['#weight'] = $i;
        $i++;
      /*} else if ((arg(0) == 'user') && (arg(1) == 'register')) {
        $form[$field_name] = $field_info;
        $form[$fieldset][$field_name]['#default_value'] = isset($contact[$field_name]) ? $contact[$field_name] : NULL;
        $form[$field_name]['#weight'] = $i;
        $form[$field_name]['fieldset'] = NULL;
        $i++;*/
      } else {

        $fieldset = $field_info['fieldset'];
        if (!isset($form[$fieldset])) {
          $form[$fieldset] = $fieldsets[$fieldset];
        }

        $form[$fieldset][$field_name] = $field_info;
        $form[$fieldset][$field_name]['#weight'] = $i;
        $form[$fieldset][$field_name]['fieldset'] = NULL;
        $form[$fieldset][$field_name]['settings-disabled'] = NULL;
        $form[$fieldset][$field_name]['settings-values'] = NULL;
        $i++;

        if (arg(0) != 'admin') {
          if (($field_name != 'amount') && ($required_fields[$field_name])) {
            $form[$fieldset][$field_name]['#required'] = TRUE;
          }
        }
        if ($field_name == 'select_amount') {
          $form[$fieldset][$field_name]['#default_value'] = ((is_numeric($amount)) && (in_array($amount, array_keys($form[$fieldset][$field_name]['#options'])))) ? $amount : 'other';
        } else if ($field_name == 'amount') {
          $form[$fieldset][$field_name]['#default_value'] = $amount;
        } else if (isset($contact[$field_name])) {
          // get data from myngp if it exists
          $form[$fieldset][$field_name]['#default_value'] = $contact[$field_name];

       	  // If checkboxes or any kind of array structure, make sure valid defaults
       	  if (is_array($form[$fieldset][$field_name]['#default_value'])) {
       	    
       	    $defs = array_filter($form[$fieldset][$field_name]['#default_value']);
       	    $defs_final = array();
       
       	    foreach ($defs as $key => $val) {
       	      if (isset($form[$fieldset][$field_name]['#options'][$key])) {
       		      $defs_final[$key] = $val;
       	      }
       	    }
       		  
       	    $form[$fieldset][$field_name]['#default_value'] = $defs_final;
       	  }
        }
      }
      if ($form[$field_name]['#type'] == 'item') {
        $form[$field_name]['#title'] = NULL;
      }
      if ($form[$field_name]['#type'] == 'hidden') {
        $form[$field_name]['#title'] = NULL;
        $form[$field_name]['#required'] = false;
        $value = variable_get('crmapi_'.$section.'_'.$field_name.'_value', false);
        $form[$field_name]['#value'] = ($value) ? $value : $form[$field_name]['#default_value'];
      }
    }

    if (isset($preContact[$field_name])) {
      // get data from facebook if it exists
      $form[$fieldset][$field_name]['#default_value'] = $preContact[$field_name];
    }
  }




  // Load interest codes from API
  if(isset($form['interest']['interestcodes'])) {
    
    $defaults = array();
    $all_fields = array_keys(crmngp_crm_fields_get_available('F', $settings['interestcodes']['prefix']));
    _debug($all_fields, 'ALL FIELDS', $settings['interestcodes']['prefix']);

    foreach($all_fields as $f) {
      $defaults[$f] = 0;
    }
    if ($contact_id != '') {
      $fields = crmngp_crm_fields_get($contact_id);
      $fields = $fields['field'];
      $fields = crmngp_crm_adjust_soap($fields, 'field');

      _debug($fields, 'Fields');
      foreach($fields as $f) {
        if (isset($defaults[$f['code']])) {
          $defaults[$f['code']] = $f['code'];
        }
      }
    }
    //$settings = variable_get('crmapi_'.$section.'_fields_settings', array('title' => t('Interest Areas'), 'prefix' => 'INT',));
    

    $form['interest']['interestcodes']['#default_value'] = array_filter($defaults);
  }

  if ($_GET['email'] != '') {
    $form['contact']['email']['#default_value'] = $_GET['email'];
  }
  elseif ((isset($user->mail)) && (arg(0) != 'admin') && (!$_REQUEST['c'])) {
    $form['contact']['email']['#default_value'] = $user->mail;
    $form['contact']['email']['#disabled'] = TRUE;
  }
  
  if ($section == 'contribute') {
    $form['reference_number'] = array('#type' => 'value', '#value' => '',);
    $form['error'] = array('#type' => 'value', '#value' => '',);
    
    if (!empty($form['payment']['select_amount'])) {
      $form['payment']['select_amount']['amount'] = $form['payment']['amount'];
      $form['payment']['select_amount']['amount']['#tree'] = TRUE;
      $form['payment']['amount'] = NULL;
    }
  }
  
  if (!empty($form['volunteer']['codes']) && !empty($_REQUEST['v'])) {
    $form['volunteer']['codes']['#default_value'] = explode(' ',$_REQUEST['v']);
  }
  
  $form['submit'] = array('#type' => 'submit', '#value' => t('Submit'), '#weight' => $i,);
  $form['submit']['#prefix'] .= "<script type='text/javascript'>btn_clicked=0;</script>";
  $form['submit']['#attributes']['onclick'] = "if (btn_clicked) return false; btn_clicked=1; this_btn=this; this.value='Please wait...'; setTimeout('this_btn.disabled=true',0); return true;";
  $form['submit']['#suffix'] = "<div class='clear-both'></div>";
  $form['#base'] = 'crmngp_ui_contact_form';

  if (!empty($options)) {
    if ($options['header'] == 0) {
      $form['header'] = NULL;
    }
    if ($options['footer'] == 0) {
      $form['footer'] = NULL;
    }
    if (!empty($options['attribution'])) {
      $form['attribution'] = array('#type'  => 'hidden', '#value' => $options['attribution']);
    }
    if (!empty($options['member'])) {
      $form['member'] = array('#type' => 'hidden', '#value' => $options['member']);
    }
    if (!empty($options['source'])) {
      $form['source']['#value'] = $options['source'];
    }
    if (!empty($options['maincode'])) {
      $form['maincode']['#value'] = $options['maincode'];
    }
  }

  return $form;
}

function crmngp_ui_contact_form_validate($form_id, $form_values, $form = array()) {
  if (!variable_get('crmngp_crm_credentials', NULL) && !variable_get('crmngp_cwp_credentials', NULL)) {
    return true;
  }
  
  $error = '';
  $preferences = variable_get('crmngp_cwp_preferences','');
  if($preferences == '') {
    variable_set('crmngp_cwp_preferences',crmngp_crm_preferences_load());
    $preferences = variable_get('crmngp_cwp_preferences','');
  }
  
  $required_fields = variable_get('crmapi_'.$form_values['section'].'_fields_required', array());
  $available_fields = crmngp_crmapi_contact_fields($form_values['section']);
  $valid = true;

  // Verify email address
  if (!eregi("^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$", $form_values['email'])) {
    form_set_error('email', t('You must enter a valid email address.'));
    $error .= "invalid email\n";
    $valid = false;
  }
  
  $international = (variable_get('crmngp_international', 0)) ? TRUE : FALSE;

  if (!$international) {
    // Verify zip code
    if (($form_values['zip'] != '') && ((!is_numeric($form_values['zip'])) || ($form_values['zip'] == 0) || ((strlen($form_values['zip']) <> 5) && (strlen($form_values['zip']) <> 9)))) {
      form_set_error('zip', t('You entered an invalid zip code.'));
      $error .= "invalid zip\n";
      $valid = false;
    }
  }

  $maxlength = ($form_values['section'] == 'contribute') ? 50 : 200;
  if (!empty($form_values['notes']) && (strlen($form_values['notes']) > $maxlength)) {
    form_set_error('notes', t('Please limit your message to %maxlength characters.', array('%maxlength' => $maxlength)));
    $error .= "invalid notes\n";
    $valid = false;
  }

  // Verify phone numbers
  $form_values['phone_home'] = str_replace(array('(', ')', '-', ' '),'',$form_values['phone_home']);
  $form_values['phone_work'] = str_replace(array('(', ')', '-', ' '),'',$form_values['phone_work']);
  $form_values['phone_mobile'] = str_replace(array('(', ')', '-', ' '),'',$form_values['phone_mobile']);
  if (($form_values['phone_home'] != '') && ((!is_numeric($form_values['phone_home'])) || ((strlen($form_values['phone_home']) != 10) && !$international) || (strlen($form_values['phone_home']) > 14) || (!ereg('^[2-9]{1}[0-9]{9}$', $form_values['phone_home']) && !$international))) {
    form_set_error('phone_home', t('You entered an invalid home phone number.'));
    $error .= "invalid home phone number\n";
    $valid = false;
  }
  if (($form_values['phone_work'] != '') && ((!is_numeric($form_values['phone_work'])) || ((strlen($form_values['phone_work']) != 10) && !$international) || (strlen($form_values['phone_home']) > 14) || (!ereg('^[2-9]{1}[0-9]{9}$', $form_values['phone_work']) && !$international))) {
    form_set_error('phone_work', t('You entered an invalid work phone number.'));
    $error .= "invalid work phone number\n";
    $valid = false;
  }
  if (($form_values['phone_work_ext'] != '') && (!is_numeric($form_values['phone_work_ext']))) {
    form_set_error('phone_work_ext', t('You entered an invalid work phone extension.'));
    $error .= "invalid work phone extension\n";
    $valid = false;
  }
  if (($form_values['phone_mobile'] != '') && ((!is_numeric($form_values['phone_mobile'])) || ((strlen($form_values['phone_mobile']) != 10) && !$international) || (strlen($form_values['phone_home']) > 14) || (!ereg('^[2-9]{1}[0-9]{9}$', $form_values['phone_mobile']) && !$international))) {
    form_set_error('phone_mobile', t('You entered an invalid mobile phone number.'));
    $error .= "invalid mobile phone number\n";
    $valid = false;
  }
  
  //print $valid;

  //print '<pre>'.print_r($form_values,1).'</pre>'; exit;

  /**
   * Contribution processing
   */
  if ($form_values['section'] == 'contribute') {
    
    //handle amount radio buttons
    if ($form_values['select_amount'] != 'other') {
      $form_values['amount'] = $form_values['select_amount'];
    } else if ($form_values['amount'] == '') {
      form_set_error('amount', t('You must enter an amount'));
    }
    $form_values['amount'] = str_replace(array('$',' '),'',$form_values['amount']);
    

    /**
     * Basic field validation
     *
     * GRE - Not sure why this is here; Drupal already checks required fields for us, 
     * this just seems to give extra error messages which are not always appropriate
     *
     * STR - This is done because crmngp_cwp_transaction_save has to happen in the
     * validation stage so that credit card processing errors can be treated as
     * validation errors by Drupal.  We therefor have to check here that all
     * required fields were sent, since Drupal's check would happen after this
     * function gets called.
     */
    foreach ($required_fields as $field => $value) {
      if (((!empty($available_fields[$field])) && ($value != '0') && (($form_values[$field] == '') || ($form_values[$field] == '0'))) && (($available_fields[$field]['#type'] != 'hidden') && ($available_fields[$field]['#type'] != 'item'))) {
        //form_set_error($field, t('The '.$field.' field is required.'));
        $valid = false;
        $error .= $field.' = '.$form_values[$field]."\n";
      }
    }
    
    //if ($valid == true) { print 'true'; exit; } else { print 'false'; exit; }


    // Retrieve contribution prefs
    if (isset($preferences['contrib']['member'])) {
      $prefs = $preferences['contrib'];
    } else {
      $prefs = $preferences['contrib'][0];
    }

    // Verify compliance checkbox
    if (!empty($required_fields['compliance']) && ($form_values['compliance'] != 1)) {
      form_set_error('compliance', t('You must confirm that these statements are true.'));
      $valid = false;
    }
    
    // Verify amount is numeric, and force into type float
    if (!is_numeric($form_values['amount'])) {
      form_set_error('amount', t('You entered an invalid amount.'));
      $valid = false;
    } else {
      $form_values['amount'] = floatval($form_values['amount']);
    }
 
    // Verify donation within Maximum /  Minimum preferences
    $amountMax = floatval($prefs['amountMax']);
    $amountMin = floatval($prefs['amountMin']);

    if ($form_values['amount'] > $amountMax) { 
      form_set_error('amount', t('You entered an amount greater than $') . $amountMax);
      $error .= 'amount greater than $' . $amountMax."\n";
      $valid = false;
    }

    if ($form_values['amount'] < $amountMin) {
      form_set_error('amount', t('You entered an amount less than $') . $amountMin);
      $error .= 'amount less than $' . $amountMin."\n";
      $valid = false;
    }
    
    // Verify and prepare recurring contributions
    if (!empty($form_values['recurring_enable']) && ($form_values['recurring_enable'] == 'true') && !empty($form_values['recurring'])) {
      if (!empty($form_values['recurring_select'])) {
        if (!empty($form_values['recurring_period'])) {
          $recurring_period = $form_values['recurring_period'];
        }
        else {
          $recurring_period = 'MONT';
        }
        
        $date_from = time();
        switch ($form_values['recurring_select']) {
          case 'contact':
            $form_values['recurring_term'] = '24';
            break;
          case 'election':
            $date_to = strtotime($prefs['ElectionDate']);
            if ($date_to < $date_from) {
              form_set_error('recurring_select', t('Election already passed'));
              $error .= "recurring - election already passed.\n";
              $valid = false;
            }
            $form_values['recurring_term'] = _crmngp_datediff($recurring_period, $date_from, $date_to, TRUE) + 1;
            break;
          case 'date':
            $date_to = mktime(0,0,0,$form_values['recurring_month'],28,$form_values['recurring_year']);
            if ($date_to < $date_from) {
              form_set_error('recurring_month', t('You must select a date in the future.'));
              $error .= "recurring date_to older than date_from.\n";
              $valid = false;
            }
            $form_values['recurring_term'] = _crmngp_datediff($recurring_period, $date_from, $date_to, TRUE) + 1;
            break;
        }
      }
      else {
        form_set_error('You must select how long you want the contribution to recurr or choose a one-time contribution', t(''));
        $error .= "no recurring period selected\n";
        $valid = false;
      }
    }

    // Verify CC Info
    if ((!is_numeric($form_values['credit_card_number'])) || (strlen($form_values['credit_card_number']) < 13) || (strlen($form_values['credit_card_number']) > 16)) {
      form_set_error('credit_card_number', t('You entered an invalid credit card number.'));
      $error .= "invalid credit card number\n";
      $valid = false;
    }
    if (!is_numeric($form_values['exp_month'])) {
      form_set_error('exp_month', t('You entered an invalid expiration month.'));
      $error .= "invalid expiration month\n";
      $valid = false;
    }
    if (!is_numeric($form_values['exp_year'])) {
      form_set_error('exp_year', t('You entered an invalid expiration year.'));
      $error .= "invalid expiration year\n";
      $valid = false;
    }
    if (($form_values['cvv'] != '') && ((!is_numeric($form_values['cvv'])) && ($form_values['cvv'] != 'NO #') && ($form_values['cvv'] != 'NO#') && ($form_values['cvv'] != 'no #') && ($form_values['cvv'] != 'no#') && ($form_values['cvv'] != 'No #') && ($form_values['cvv'] != 'No#'))) {
      form_set_error('cvv', t('You entered an invalid security code.'));
      $error .= "invalid cvv\n";
      $valid = false;
    }
    //$valid = false;
    //$error = 'test';
    
    if ($valid == false) {
      form_set_value(array('#parents' => array('error')), $error);
    }
    
    //dsm($form_values);
    
    /**
     * Submit transaction in the validation stage so any failures can be
     * returned as validation errors thus allowing the user to resubmit.
     */
    if ($valid == true) {
      //call postVerisignTransaction - status should = 0 - vendor result should eqal zero
      $result = crmngp_cwp_transaction_save($form_values);
      switch ($result['code']) {
        case 0:
          if (empty($result['referencenumber'])) {
            form_set_error('credit_card_number', t('Unknown error.  The site administrator will be notified.  Please try again later.'));
            $from = variable_get('site_mail', ini_get('sendmail_from'));
            $subject = 'CRMNGP: CWP returned 0 without reference number';
            drupal_mail('crmngp-admin', 'icampaignteam@ngpsoftware.com', $subject, t("Received an error from CWP:\n\n").print_r($result,1), $from);
            break;
          }
          drupal_set_message(t('Your confirmation number is %number', array('%number' => $result['referencenumber'])));
          $form['reference_number']['#parents'] = array('reference_number');
          form_set_value(array('#parents' => array('reference_number')), $result['referencenumber']);
          break;
        case 1:
          form_set_error('credit_card_number', t('Sorry, an error has occurred.  The site administrator will be notified.  Please try again later.'));
          $from = variable_get('site_mail', ini_get('sendmail_from'));
          $subject = 'CRMNGP: Verisign Authentication Failed';
          drupal_mail('crmngp-admin', $from.', icampaignteam@ngpsoftware.com', $subject, t("Received 'Authentication Failed' from Verisign.  Please check the server settings.\n\n").print_r($result,1), $from);
          break;
        case 3:
          form_set_error('amount', t('Invalid transaction type.'));
          break;
        case 4:
          form_set_error('amount', t('You entered an invalid amount.'));
          break;
        case 5:
          form_set_error('amount', t('Invalid merchant information.'));
          break;
        case 6:
          form_set_error('amount', t('Missing required information.'));
          break;
        case 7:
          form_set_error('amount', t('Field format error.  The site administrator will be notified.  Please try again later.'));
          $from = variable_get('site_mail', ini_get('sendmail_from'));
          $subject = 'CRMNGP: Verisign Field Format Error';
          drupal_mail('crmngp-admin', $from.', icampaignteam@ngpsoftware.com', $subject, t("Received 'Field Format Error' from Verisign.\n\n").print_r($result,1), $from);
          break;
        case 12:
          form_set_error('credit_card_number', t('Your contribution was declined.'));
          break;
        case 13:
          form_set_error('credit_card_number', t('We could not process your transaction. Please contact your card issuer.  Thank you.'));
          break;
        case 23:
          form_set_error('credit_card_number', t('You entered an invalid credit card number.'));
          break;
        case 24:
          form_set_error('exp_year', t('Your card has expired.'));
          break;
        case 25:
          form_set_error('credit_card_number', t('The campaign does not accept this card type.'));
          break;
        case 30:
          form_set_error('credit_card_number', t('You have submitted a duplicate transaction.'));
          break;
        case 36:
          form_set_error('credit_card_number', t('Your contribution was declined.'));
          break;
        case 50:
          form_set_error('credit_card_number', t('Insufficient Funds.'));
          break;
        case 109:
          form_set_error('credit_card_number', t('Timeout waiting for Host response.  Please try again later.'));
          break;
        case 112:
          form_set_error('credit_card_number', t('Address Verification System check failed.  Please check your address and try again.'));
          break;
        case 1000:
          if ($result['message'] == 'Generic processor error: 57-Terminal is not programmed for this service - call Customer Support') {
            form_set_error('credit_card_number', t('You have entered an invalid card.  Please try another one.'));
          }
          else {
            form_set_error('credit_card_number', t('Unknown error.  The site administrator will be notified.  Please try again later.'));
            $from = variable_get('site_mail', ini_get('sendmail_from'));
            $subject = 'CRMNGP: Verisign Generic Host Error';
            drupal_mail('crmngp-admin', 'api@ngpsoftware.com', $subject, t("Received 'Generic Host Error' from Verisign.\n\n").print_r($result,1), $from);
          }
          break;
        default:
          form_set_error('credit_card_number', t('Unknown error.  The site administrator will be notified.  Please try again later.'));
          $from = variable_get('site_mail', ini_get('sendmail_from'));
          $subject = 'CRMNGP: CWP Error';
          drupal_mail('crmngp-admin', 'api@ngpsoftware.com', $subject, t("Received an error from CWP:\n\n").print_r($result,1), $from);
          break;
      }
      //print '<pre>'.print_r($result,1).'</pre>'; exit;
    }
  }
}

/**
 * _submit function for our crmngp_ui_individual_form(). Merely
 * a wrapper around the actual NGPAPI function that adds users.
 */
function crmngp_ui_contact_form_submit($form_id, $form_values) {
  if ($form_values['section'] == 'contribute') {
    $form_values['credit_card_number'] = NULL;
    $form_values['exp_year'] = NULL;
    $form_values['exp_month'] = NULL;
    
    //handle amount radio buttons
    if ($form_values['select_amount'] != 'other') {
      $form_values['amount'] = $form_values['select_amount'];
    }
    $form_values['amount'] = str_replace(array('$',' '),'',$form_values['amount']);
    
    if (empty($form_values['reference_number'])) {
      db_query("INSERT INTO {crmngp_log} (`timestamp`, `values`) VALUES ('%d', '%s')", time(), serialize($form_values));
      
      $from = variable_get('site_mail', ini_get('sendmail_from'));
      $subject = 'CRMNGP: Validation Error - '.variable_get('site_name', '');
      drupal_mail('crmngp-admin', 'api@ngpsoftware.com', $subject, $form_values['error'], $from);
      
      drupal_set_message('Unknown error.  The site administrator will be notified.  Please try again later.', 'error');
      
      return false;
    }
    else if (!empty($form_values['place_of_business'])) {
      $contact_id = crmngp_crm_contribution_get_reference($form_values['reference_number']);
      if (!empty($contact_id)) {
        $contact = array(
          'contact_id' => $contact_id,
          'place_of_business' => $form_values['place_of_business'],
        );
        $new_id = crmngp_crm_contact_save_va($contact);
        if ($contact_id != $new_id) {
          //dsm("contactSetVA result mismatch: $contact_id != $new_id");
        }
      }
      else {
        //dsm('contributionGetByReference failed');
      }
    }
  }
  if (variable_get('crmngp_email', 0)) {
    drupal_mail('crmngp-admin', variable_get('crmngp_email_destination', variable_get('site_mail', '')), 'Received a '.$form_values['section'].' form submission', t("Received the following form submission:\n\n").print_r($form_values,1), variable_get('site_mail', ''));
  }

  $settings = variable_get('crmapi_'.$section.'_fields_settings', crmngp_default_settings());

  //drupal_set_message('<pre>'.print_r($form_values,1).'</pre>');
  if ($form_values['section'] != 'contribute') {
    /*/sanitize data to prevent SQL injections in COO
    foreach ($form_values as $key => $value) {
      if (!is_array($value)) {
        $form_values[$key] = _crmngp_sanitize($value);
      }
    }*/
  
    //drupal_set_message('<pre>'.print_r($form_values, 1).'</pre>');
  
    //fix mailname
    $form_values['mail_name'] = $form_values['first_name'].' '.$form_values['last_name'];
  
    //don't want codes, notes, or contribution data going into the contact record
    $codes = $form_values['codes'];
    $notes = $form_values['notes'];
    $amount = $form_values['amount'];
    $maincode = $form_values['maincode'];
    $sectioncode = $form_values['sectioncode'];

    // TODO: Replace these with unset() ?
    $form_values['codes'] = NULL;
    $form_values['notes'] = NULL;
    $form_values['amount'] = NULL;
    $form_values['credit_card_number'] = NULL;
    $form_values['exp_year'] = NULL;
    $form_values['exp_month'] = NULL;
    $form_values['maincode'] = NULL;
    $form_values['sectioncode'] = NULL;
    
    $contact_id = crmngp_crm_contact_save($form_values);
    
    global $user;
    
    if (!empty($codes)) {
      foreach ($codes as $code_name => $code_value) {
        if ($code_value != '0') {
          $volunteer_id = crmngp_crm_volunteer_save($contact_id, $code_value, $notes);
          //$notes = ''; //notes only needs to appear for the first volunteer option
        }
      }
    }
    
    if (!empty($maincode) || !empty($sectioncode)) {
      $codes = array();
      if (!empty($maincode)) {
        $codes[$maincode] = $maincode;
      }
      if (!empty($sectioncode)) {
        $codes[$sectioncode] = $sectioncode;
      }
      crmngp_crm_contact_save_codes(array('contact_id' => $contact_id, 'codes' => $codes));
    }
    
    if (!empty($notes)) {
      crmngp_crm_note_save($contact_id, array('text' => $notes));
    }

    // See if interest areas was present on the form
    $visible_fields = variable_get('crmapi_'.$section.'_fields', array());
    if(in_array('interestcodes',array_keys($visible_fields))) {
      
      $prefix = $settings['interestcodes']['prefix'];

      _debug($form_values['interestcodes'], 'submitted');
      _debug(crmngp_crm_fields_get_available('F', $prefix), 'Available Fields');
      _debug($settings, 'Settings');

      crmngp_crm_fields_update($contact_id, 
        array_keys(crmngp_crm_fields_get_available('F',$prefix)),
        array_filter($form_values['interestcodes']),
        'OL' . $settings['interestcodes']['prefix']
      );
    }
  }

  if ($form_values['section'] == 'user') {
    return $contact_id;
  } else {
    if (!empty($contact_id) && ($user->uid != '0')) {
      user_save($user, array('crmapi_contact' => $contact_id));
    }
    if ($form_values['section'] == 'subscribe' || $form_values['section'] == 'volunteer') {
      crmngp_email_confirmation($form_values);
    }
    return 'crmapi/thanks/'.$form_values['section'];
  }
}


function crmngp_email_confirmation($form_values) {
  $preferences = variable_get('crmngp_cwp_preferences','');
  if($preferences == '') {
    variable_set('crmngp_cwp_preferences', crmngp_crm_preferences_load());
    $preferences = variable_get('crmngp_cwp_preferences', null);
  }
  //  mail('gengel@citidc.com, greglavallee@gmail.com', "[debug] NGP Mailing Prefs", print_r($preferences,true));

  $campaignName = $preferences['campaignInfo']['name'];
  $tree = ($form_values['section'] == 'volunteer') ? 'volunteer' : 'emailSignUp';
  $preferences = $preferences[$tree];

  //dsm($preferences);
  if($preferences['confirmationEmails']['enabled'] == 'True') {
    $mailkey = $form_values['section'] . ' confirmationEmail';
    $to = $form_values['email'];
    $from = $preferences['fromEmailAddress'];
    $name = $form_values['first_name'];
    $subject = $preferences['confirmationEmails']['subject'];

    // Create Body
    $body = str_replace("\n", '<br/>', html_entity_decode($preferences['confirmationEmails']['message']));
    $body = str_replace('{0}', $name, html_entity_decode($preferences['confirmationEmails']['messagePrefix'])) . "\r\n<br/>" . $body . "\r\n<br/><br/>";
    if ($preferences['confirmationEmails']['includeFooter'] == 'True') {
      $body .= '<br/><table border="1px" bordercolor="grey" width="100%"><tr><td align="center">';
      $body .= variable_get('crmngp_email_footer', t('Paid for and Authorized by '));
      $body .= $campaignName . "</td></tr></table>";
    }
    $html_mime = array(
      'MIME-Version' => '1.0',
      'Content-Type' => 'text/html; charset=UTF-8; format=flowed',
      'Content-Transfer-Encoding' => '8Bit',
      'X-Mailer' => 'Drupal'
    );
    if (!drupal_mail($mailkey, $to, $subject, $body, $from, $html_mime)) {
      watchdog('mail', 'Failed to send ' . $mailkey . ' email to ' . $to, WATCHDOG_ERROR, 'crmapi/' . $form_values['section']);
    }
  }
}

function theme_crmngp_ui_contact_form($form) {
  $layout = 'onecol';
  if ($form['payment'] || $form['compliance'] || $form['volunteer'] || $form['notes']) {
    $layout = 'twocol';
  }
  
  $output = '';
  $output .= drupal_render($form['header']);
  $output .= '<div id="crmngp-'.$form['section']['#value'].'" class="crmngp-form crmngp-'.$layout.'">';
  $output .= '<div id="crmngp-left" class="crmngp-column"><div class="inner">';
  $output .= drupal_render($form['contact']);
  $output .= drupal_render($form['employment']);

  if ($form['interest']) {
    $output .= drupal_render($form['interest']);
  }

  if ($form['payment'] || $form['compliance'] || $form['volunteer'] || $form['notes']) {
    $output .= '</div></div>';
    $output .= '<div id="crmngp-right" class="crmngp-column"><div class="inner">';
    $output .= drupal_render($form['payment']);
    $output .= drupal_render($form['compliance']);
    $output .= drupal_render($form['volunteer']);
    $output .= drupal_render($form['notes']);
  }
  $output .= '</div></div>';
  $output .= '<div class="clear-both"></div>';
  $output .= '<div id="crmngp-footer">';
  $output .= drupal_render($form['submit']);
  $output .= '</div>';
  $output .= '</div>';
  if ($form['crmngp_compliance']['#value'] != '') {
    $output .= drupal_render($form['crmngp_disclaimer']);
  }
  $output .= '<div class="clear-both"></div>';
  if ($form['footer']['#value'] != '') {
    $output .= '<div id="crmngp-disclaimer">'.drupal_render($form['footer']).'</div>';
  }
  $output .= drupal_render($form);
  return $output;
}

function phptemplate_form_element($element, $value) {
  $output  = '<div class="form-item form-item-'.$element['#id'].'">'."\n";
  $required = !empty($element['#required']) ? '<span class="form-required" title="'. t('This field is required.') .'">*</span>' : '';

  if (!empty($element['#title'])) {
    $title = $element['#title'];
    if (!empty($element['#id'])) {
      $output .= ' <label for="'. $element['#id'] .'">'. t('!title: !required', array('!title' => filter_xss_admin($title), '!required' => $required)) ."</label>\n";
    }
    else {
      $output .= ' <label>'. t('!title: !required', array('!title' => filter_xss_admin($title), '!required' => $required)) ."</label>\n";
    }
  }

  $output .= " $value\n";

  if (!empty($element['#description'])) {
    $output .= ' <div class="description">'. $element['#description'] ."</div>\n";
  }

  $output .= "</div>\n";

  return $output;
}

function crmngp_ui_block($op = 'list', $delta = 0, $edit = array()) {
  if ($op == 'list') {
    $blocks[0] = array('info' => t('CRMNGP: Subscribe'), 'weight' => 0, 'enabled' => 0, 'region' => 'left');
    $blocks[1] = array('info' => t('CRMNGP: Contribute'), 'weight' => 0, 'enabled' => 0, 'region' => 'left');
    return $blocks;
  }
  else if ($op == 'configure') {
    switch($delta) {
      case 0:
        $form['subscribe_inline'] = array(
          '#type' => 'checkbox',
          '#title' => t('Inline field titles'),
          '#return_value' => 1,
          '#default_value' => variable_get('crmngp_subscribe_inline', 1),
          '#description' => t('Display the field titles inside the field.  They\'ll disappear when a user clicks into the field'),
        );
        $form['subscribe_name'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show name fields'),
          '#return_value' => 1,
          '#default_value' => variable_get('crmngp_subscribe_name', 0),
          '#description' => t('Checking this will show the first and last name fields in the subscription block'),
        );
        $form['subscribe_zipcode'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show zipcode field'),
          '#return_value' => 1,
          '#default_value' => variable_get('crmngp_subscribe_zipcode', 1),
          '#description' => t('Unchecking this will hide the zipcode field in the subscription block'),
        );
        break;
      case 1:
        $form['contribute_inline'] = array(
          '#type' => 'checkbox',
          '#title' => t('Inline field titles'),
          '#return_value' => 1,
          '#default_value' => variable_get('crmngp_contribute_inline', 1),
          '#description' => t('Display the field titles inside the field.  They\'ll disappear when a user clicks into the field'),
        );
        break;
    }
    return $form;
  }
  else if ($op == 'save') {
    switch($delta) {
      case 0:
        variable_set('crmngp_subscribe_inline', $edit['subscribe_inline']);
        variable_set('crmngp_subscribe_name', $edit['subscribe_name']);
        variable_set('crmngp_subscribe_zipcode', $edit['subscribe_zipcode']);
        break;
      case 1:
        variable_set('crmngp_contribute_inline', $edit['contribute_inline']);
        break;
    }
  }
  else if ($op == 'view') {
    switch($delta) {
      case 0:
        $block = array('subject' => t('Subscribe'),
          'content' => drupal_get_form('crmngp_ui_form_subscribe'));
        break;
      case 1:
        $block = array('subject' => t('Contribute'),
          'content' => drupal_get_form('crmngp_ui_form_contribute'));
        break;
    }
    return $block;
  }
}

/**
 * Subscription Form
 */
function crmngp_ui_form_subscribe() {
  $form = array();

  $preferences = variable_get('crmngp_cwp_preferences', '');
  if (($preferences == '') || !is_array($preferences)) {
    $preferneces = crmngp_crm_preferences_load();
    if (is_array($preferences)) {
      variable_set('crmngp_cwp_preferences', $preferences);
    }
    $preferences = variable_get('crmngp_cwp_preferences', '');
  }
  
  if (!empty($preferences['general'])) {
    $form['subscribe_maincode']           = array('#title' => 'Maincode', '#type'  => 'hidden', '#default_value' => $preferences['general']['mainCodes']['code']);
    $form['subscribe_sectioncode']        = array('#title' => 'Section Code', '#type'  => 'hidden', '#default_value' => $preferences['general']['emailMainCode']);
  }
  if (variable_get('crmngp_subscribe_name', 0)) {
    $form['subscribe_first_name'] = array(
      '#type' => 'textfield',
      '#title' => t('First Name'),
      '#default_value' => '',
      '#size' => 20,
      '#maxlength' => 256,
      '#description' => NULL,
      '#attributes' => NULL,
      '#required' => FALSE,
    );
    $form['subscribe_last_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Last Name'),
      '#default_value' => '',
      '#size' => 20,
      '#maxlength' => 256,
      '#description' => NULL,
      '#attributes' => NULL,
      '#required' => FALSE,
    );
    if (variable_get('crmngp_subscribe_inline', 1)) {
      $form['subscribe_first_name']['#title'] = '';
      $form['subscribe_first_name']['#default_value'] = t('First Name');
      $form['subscribe_first_name']['#attributes'] = array('onclick' => "if(this.value=='". t('First Name') ."'){this.value='';}");
      $form['subscribe_last_name']['#title'] = '';
      $form['subscribe_last_name']['#default_value'] = t('Last Name');
      $form['subscribe_last_name']['#attributes'] = array('onclick' => "if(this.value=='". t('Last Name') ."'){this.value='';}");
    }
  }
  $form['subscribe_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Email Address'),
    '#default_value' => '',
    '#size' => 20,
    '#maxlength' => 256,
    '#description' => NULL,
    '#required' => true,
  );
  if (variable_get('crmngp_subscribe_inline', 1)) {
    $form['subscribe_email']['#title'] = '';
    $form['subscribe_email']['#default_value'] = t('Email Address');
    $form['subscribe_email']['#attributes'] = array('onclick' => "if(this.value=='". t('Email Address') ."'){this.value='';}");
  }
  if (variable_get('crmngp_subscribe_zipcode', 1)) {
    $form['subscribe_zip'] = array(
      '#type' => 'textfield',
      '#title' => t('Zip Code'),
      '#default_value' => '',
      '#size' => 7,
      '#maxlength' => 10,
      '#description' => NULL,
      '#required' => true,
    );
    if (variable_get('crmngp_subscribe_inline', 1)) {
      $form['subscribe_zip']['#title'] = '';
      $form['subscribe_zip']['#default_value'] = t('Zip Code');
      $form['subscribe_zip']['#attributes'] = array('onclick' => "if(this.value=='". t('Zip Code') ."'){this.value='';}");
    }
  }
  $form['subscribe_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Subscribe'),
  );

  return $form;
}

/**
 * Validate Subscription Form
 */
function crmngp_ui_form_subscribe_validate($form_id, &$form_values) {
  // Verify email address
  if (!eregi("^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$", $form_values['subscribe_email'])) {
    form_set_error('subscribe_email', t('You must enter a valid email address.'));
    $valid = false;
  }

  // Verify zip code
  if (($form_values['subscribe_zip'] != '') && ((!is_numeric($form_values['subscribe_zip'])) || ((strlen($form_values['subscribe_zip']) <> 5) && (strlen($form_values['subscribe_zip']) <> 9)))) {
    form_set_error('subscribe_zip', t('You entered an invalid zip code.'));
    $valid = false;
  }
}

/**
 * Submit Subscription Form
 */
function crmngp_ui_form_subscribe_submit($form_id, $form_values) {
  $form_values['contact_type'] = 'I';
  $form_values['first_name'] = (!empty($form_values['subscribe_first_name']) && ($form_values['subscribe_first_name'] != 'First Name')) ? $form_values['subscribe_first_name'] : NULL;
  $form_values['last_name'] = (!empty($form_values['subscribe_last_name']) && ($form_values['subscribe_last_name'] != 'Last Name')) ? $form_values['subscribe_last_name'] : NULL;
  $form_values['zip'] = $form_values['subscribe_zip'];
  $form_values['email'] = $form_values['subscribe_email'];
  
  $maincode = $form_values['subscribe_maincode'];
  $sectioncode = $form_values['subscribe_sectioncode'];
  
  $form_values['subscribe_first_name'] = NULL;
  $form_values['subscribe_last_name'] = NULL;
  $form_values['subscribe_email'] = NULL;
  $form_values['subscribe_zip'] = NULL;
  $form_values['subscribe_maincode'] = NULL;
  $form_values['subscribe_sectioncode'] = NULL;
  
  $contact_id = crmngp_crm_contact_save($form_values);
  
  if (!empty($maincode) || !empty($sectioncode)) {
    $codes = array();
    if (!empty($maincode)) {
      $codes[$maincode] = $maincode;
    }
    if (!empty($sectioncode)) {
      $codes[$sectioncode] = $sectioncode;
    }
    crmngp_crm_contact_save_codes(array('contact_id' => $contact_id, 'codes' => $codes));
  }
  
  $_SESSION['contact_id'] = $contact_id;
  return 'crmapi/subscribe';
}

/**
 * Contribution Form
 */
function crmngp_ui_form_contribute() {
  $form = array();

  $form['contribute_amount'] = array(
    '#type' => 'textfield',
    '#title' => t('Amount'),
    '#default_value' => '',
    '#size' => 7,
    '#maxlength' => 256,
    '#description' => NULL,
    '#required' => true,
  );
  if (variable_get('crmngp_ui_contribute_inline', 1)) {
    $form['contribute_amount']['#title'] = '';
    $form['contribute_amount']['#default_value'] = t('Amount');
    $form['contribute_amount']['#attributes'] = array('onclick' => "if(this.value=='". t('Amount') ."'){this.value='';}");
  }
  $form['contribute_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Contribute'),
  );

  return $form;
}

/**
 * Submit Subscription Form
 * Validate Contribution Form
 */
function crmngp_ui_form_contribute_validate($form_id, &$form_values) {
  // Verify zip code
  if (($form_values['contribute_amount'] != '') && (!is_numeric($form_values['contribute_amount']))) {
    form_set_error('contribute_amount', t('You entered an invalid amount.'));
    $valid = false;
  }
}

/**
 * Submit Contribution Form
 */
function crmngp_ui_form_contribute_submit($form_id, $form_values) {
  
  $server_name = explode('.', $_SERVER['SERVER_NAME']);
  $tld = ($server_name[0] == 'www') ? $server_name[1] : $server_name[0];
  $secure_server = 'https://'.$tld.'.ngphost.com';
  drupal_goto($secure_server.'/crmapi/contribute', 'a='.$form_values['contribute_amount']);
}
